//  test_student_t.cpp

// Copyright Paul A. Bristow 2006.

// Use, modification and distribution are subject to the
// Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt
// or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifdef _MSC_VER
#  pragma warning(disable: 4127) // conditional expression is constant.
#  pragma warning(disable: 4100) // unreferenced formal parameter.
#  pragma warning(disable: 4189) // local variable is initialized but not referenced.
#  pragma warning(disable: 4512) // assignment operator could not be generated.
#  if !(defined _SCL_SECURE_NO_DEPRECATE) || (_SCL_SECURE_NO_DEPRECATE == 0)
#    pragma warning(disable: 4996) // 'std::char_traits<char>::copy' was declared deprecated.
// #define _SCL_SECURE_NO_DEPRECATE = 1 // avoid C4996 warning.
#  endif
#endif

#define BOOST_MATH_THROW_ON_DOMAIN_ERROR
#include <boost/math/special_functions/beta.hpp>
using boost::math::ibeta; // ibeta(a, b, x)

#include <iostream>
using std::cout;
using std::endl;
using std::setprecision;
#include <limits>
using std::numeric_limits;
#include <cassert>

// class students_t To go into student_t_class.hpp

template <typename T>
class students_t
{
public:
  explicit students_t(T degrees_of_freedom) : m_degrees_of_freedom (degrees_of_freedom)
  { // Need to check degrees_of_freedom > 0
  }
  // Use of explicit forces user to provide value for degrees_of_freedom.
  // T probability(T x); // Probability density function (pdf)

  T probability(T x)  // Probability density function (pdf)
  { // TODO.
    return numeric_limits<T>::quiet_NaN();
  }
  //T probability(T l, T u); // Range of integration or summation.
  // cumulative density function cdf.

  T probability(T l, T u) // Range of integration or summation.
  {
    using boost::math::ibeta; // ibeta(a, b, x)
    if (l == -numeric_limits<T>::infinity())
    { // integral -infinity to u.  TODO Check this is right!
      T z = m_degrees_of_freedom / (m_degrees_of_freedom + u * u);
      return ibeta(m_degrees_of_freedom / 2, static_cast<T>(0.5), z) / 2;
    }
    else if (u == +numeric_limits<T>::infinity())
    { // complementary integral l to +infinity
      return ibeta(static_cast<T>(0.5), m_degrees_of_freedom / 2, l) / 2; // TODO get this right!
    };
    //else
    // Integral l to u not implemented yet.
    return numeric_limits<T>::quiet_NaN();
  }

   T probability_lower_tail(T l) // Lower tail -infinity to l.
  {
    using boost::math::ibeta; // ibeta(a, b, x)
      T z = m_degrees_of_freedom / (m_degrees_of_freedom + u * u);
      return ibeta(m_degrees_of_freedom / 2, static_cast<T>(0.5), z) / 2;
    }

   T probability_upper_tail(T l) // Upper tail u to +infinity
  {
    using boost::math::ibeta; // ibeta(a, b, x)
   T z = m_degrees_of_freedom / (m_degrees_of_freedom + u * u);
    return ibeta(static_cast<T>(0.5), m_degrees_of_freedom / 2, l) / 2; // TODO get this right!
  }

  T quantile(T probability); // t for cumulative probability & degrees_of_freedom.
  // Also known as inverse cumulative distribution function or percentile.

  T degrees_of_freedom()
  {
    return m_degrees_of_freedom;
  }

  static T degrees_of_freedom_quantile(T quantile, T probability);
  // Predicts degrees of freedom necessary for t and cumulative probability.
  // What should this be called?

  // Functions below return NaN, if undefined.
  T mean()
  { // 
    if (m_degrees_of_freedom == 1)
    { // Undefined.
      return numeric_limits<T>::quiet_NaN();
    }
    return 0; // for m_degrees_of_freedom > 1
  }

  T variance()
  { 
    if (m_degrees_of_freedom <= 2)
    {
     return numeric_limits<T>::infinity();
    }
    return m_degrees_of_freedom / (m_degrees_of_freedom -2);
  }

  T skewness()
  {
    if (m_degrees_of_freedom <= 3)
    {
     return numeric_limits<T>::quiet_NaN();
    }
    return 0;
  }
  //T kurtosis() const;???
  T m_degrees_of_freedom;
}; // template <typename T> class students_t



int main(int, char* [])
{
  double degrees_of_freedom = 20;
  students_t<double> my_students_t(degrees_of_freedom);
  assert(my_students_t.degrees_of_freedom() == degrees_of_freedom);

  double t = 1.5;
  double pdf = my_students_t.probability(t); // Probability density function.
  // Or is the danger too great that users will assume this is cdf -infinity to t????

  // double pdf = my_students_t.density(t); // Probability density function.

  double cdf = my_students_t.probability(-numeric_limits<double>::infinity(), t); // cumulative -infinity to t = 1.5
  //double prob2 = 2 * my_students_t.probability(t, +numeric_limits<double>::infinity()); // 2-tailed.

  assert(my_students_t.mean() == 0);

  return 0;
} // int main(int, char* [])

/*

Output:

------ Build started: Project: students_t_class, Configuration: Debug Win32 ------
Compiling...
students_t_class.cpp
Linking...
Autorun "i:\boost-06-05-03-1300\libs\math\test\Math_test\debug\students_t_class.exe"
Build Time 0:03
Build log was saved at "file://i:\boost-06-05-03-1300\libs\math\test\Math_test\students_t_class\Debug\BuildLog.htm"
students_t_class - 0 error(s), 0 warning(s)
========== Build: 1 succeeded, 0 failed, 0 up-to-date, 0 skipped ==========



*/

